# coding: utf-8
from __future__ import unicode_literals, absolute_import

import re

from host_user_override import conf


HOST_SUB_REGEXP = re.compile(conf.HOST_SUB_REGEXP, re.UNICODE)


def get_original_host(host):
    return re.sub(HOST_SUB_REGEXP, '', host, 1)



# coding: utf-8
from __future__ import unicode_literals, absolute_import

import re

from django.contrib.auth import get_user_model
from django.core.exceptions import PermissionDenied
from django.http import HttpResponseRedirect, HttpResponsePermanentRedirect
from django.template.loader import render_to_string
from django.utils.encoding import force_text

from host_user_override import conf
from host_user_override.utils import get_original_host


_HTML_TYPES = ('text/html', 'application/xhtml+xml')

REDIRECT_RESPONSE_CLASS = HttpResponseRedirect
if conf.PERMANENT_REDIRECT:
    REDIRECT_RESPONSE_CLASS = HttpResponsePermanentRedirect


class HostUserOverrideMiddleware(object):
    """
    Overrides current user if host is like <id>.user.<host> and AuthenticationMiddleware
    tells current user is superuser.
    """
    def __init__(self, get_response):
        self.get_response = get_response
        self.regexp = re.compile(conf.HOST_REGEXP)

    def get_user_id(self, host):
        match = self.regexp.match(host)
        if match is not None:
            groups = match.groups()
            if groups:
                return int(groups[0])
        return None

    def __call__(self, request):
        overridden = False
        original_user = request.user

        host = request.META.get('HTTP_HOST', '')
        user_id = self.get_user_id(host)

        user = request.user
        User = get_user_model()
        if user_id is not None and user_id != request.user.pk:
            try:
                user = User.objects.exclude(is_superuser=True).get(pk=user_id)
            except User.DoesNotExist:
                raise PermissionDenied

        if conf.PERMISSION_CHECK(request.user, user):
            request.user = user
            overridden = True
        else:
            # Redirect non-superusers to original domain
            # Probably should be 302 redirect, but we need permanent for SEO
            if user_id is not None:
                original_host = get_original_host(host)
                return REDIRECT_RESPONSE_CLASS(u'{scheme}://{host}{path}'.format(
                    scheme=request.scheme, host=original_host, path=request.path
                ))

        response = self.get_response(request)

        # Check for responses where the toolbar can't be inserted.
        content_encoding = response.get('Content-Encoding', '')
        content_type = response.get('Content-Type', '').split(';')[0]
        if any((getattr(response, 'streaming', False),
                'gzip' in content_encoding,
                content_type not in _HTML_TYPES)):
            return response

        if overridden:
            # Inject top banner
            content = force_text(response.content, encoding=response.charset)
            insert_after = '<body>'
            pattern = re.escape(insert_after)
            bits = re.split(pattern, content, flags=re.IGNORECASE)
            if len(bits) > 1:
                bits[1] = render_to_string('host_user_override/banner.html', request=request, context={
                    'user': user,
                    'original': original_user,
                }) + bits[1]
                response.content = insert_after.join(bits)
                if response.get('Content-Length', None):
                    response['Content-Length'] = len(response.content)

        return response



# coding: utf-8
from __future__ import unicode_literals, absolute_import

from django.conf import settings


prefix = 'HOSTUSEROVERRIDE_'


def get_setting(name, default=None):
    return getattr(settings, prefix+name, default)


def permission_check(current_user, desired_user):
    return current_user.is_superuser


HOST_REGEXP = get_setting('HOST_REGEXP', r'(\d+)\.user\..+')

HOST_SUB_REGEXP = get_setting('HOST_SUB_REGEXP', r'\d+\.user\.')

REDIRECT_URL_FORMAT = get_setting('REDIRECT_URL_FORMAT', 'http://{user_id}.user.{host}/')

PERMANENT_REDIRECT = get_setting('HOST_SUB_REGEXP', False)

PERMISSION_CHECK = get_setting('PERMISSION_CHECK', permission_check)



# coding: utf-8
from __future__ import unicode_literals, absolute_import

from django import template

from host_user_override import conf
from host_user_override.utils import get_original_host


register = template.Library()


@register.simple_tag(takes_context=True, name='get_original_host')
def get_original_host_tag(context):
    request = context['request']
    return get_original_host(request.get_host())


@register.simple_tag(takes_context=True)
def get_login_link(context, user):
    request = context['request']
    host = get_original_host(request.get_host())
    return conf.REDIRECT_URL_FORMAT.format(host=host, user_id=user.pk)


@register.filter()
def has_permission_to_be(user, desired_user):
    return conf.PERMISSION_CHECK(user, desired_user)
